function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// DOM Utils
import getElementClientRect from './dom-utils/getElementClientRect';
import listScrollParents from './dom-utils/listScrollParents';
import getWindow from './dom-utils/getWindow';
import getNodeScroll from './dom-utils/getNodeScroll';
import getOffsetParent from './dom-utils/getOffsetParent';
import getCommonTotalScroll from './dom-utils/getCommonTotalScroll'; // Pure Utils

import unwrapJqueryElement from './utils/unwrapJqueryElement';
import orderModifiers from './utils/orderModifiers';
import expandToHashMap from './utils/expandToHashMap';
import computeOffsets from './utils/computeOffsets';
import format from './utils/format';
import debounce from './utils/debounce';
import validateModifiers from './utils/validateModifiers'; // Default modifiers

import * as modifiers from './modifiers/index';
var defaultModifiers = Object.values(modifiers);
var INVALID_ELEMENT_ERROR = 'Popper: Invalid `%s` argument provided to Popper, it must be either a valid DOM element or a jQuery-wrapped DOM element, you provided `%s`';
var INFINITE_LOOP_ERROR = 'Popper: An infinite loop in the modifiers cycle has been detected! The cycle has been interrupted to prevent a browser crash.';

var areValidElements = function areValidElements(a, b) {
  return a instanceof Element && b instanceof Element;
};

var defaultOptions = {
  placement: 'bottom',
  eventListeners: {
    scroll: true,
    resize: true
  },
  modifiers: [],
  strategy: 'absolute'
};

var Popper =
/*#__PURE__*/
function () {
  function Popper(reference, popper, options) {
    var _this = this;

    if (options === void 0) {
      options = defaultOptions;
    }

    _defineProperty(this, "state", {
      placement: 'bottom',
      orderedModifiers: [],
      options: defaultOptions,
      modifiersData: {}
    });

    _defineProperty(this, "update", debounce(function () {
      return new Promise(function (success, reject) {
        _this.forceUpdate();

        success(_this.state);
      });
    }));

    // Unwrap `reference` and `popper` elements in case they are
    // wrapped by jQuery, otherwise consume them as is
    this.state.elements = {
      reference: unwrapJqueryElement(reference),
      popper: unwrapJqueryElement(popper)
    };
    var _this$state$elements = this.state.elements,
        referenceElement = _this$state$elements.reference,
        popperElement = _this$state$elements.popper; // Store options into state

    this.state.options = _extends({}, defaultOptions, {}, options); // Don't proceed if `reference` or `popper` are invalid elements

    if (!areValidElements(referenceElement, popperElement)) {
      return;
    }

    this.state.scrollParents = {
      reference: listScrollParents(referenceElement),
      popper: listScrollParents(popperElement)
    }; // Order `options.modifiers` so that the dependencies are fulfilled
    // once the modifiers are executed

    this.state.orderedModifiers = orderModifiers([].concat(defaultModifiers, this.state.options.modifiers)) // Apply user defined preferences to modifiers
    .map(function (modifier) {
      return _extends({}, modifier, {}, _this.state.options.modifiers.find(function (_ref) {
        var name = _ref.name;
        return name === modifier.name;
      }));
    }); // Validate the provided modifiers so that the consumer will get warned
    // of one of the custom modifiers is invalid for any reason

    if (process.env.NODE_ENV !== "production") {
      validateModifiers(this.state.options.modifiers);
    } // Modifiers have the opportunity to execute some arbitrary code before
    // the first update cycle is ran, the order of execution will be the same
    // defined by the modifier dependencies directive.
    // The `onLoad` function may add or alter the options of themselves


    this.state.orderedModifiers.forEach(function (_ref2) {
      var onLoad = _ref2.onLoad,
          enabled = _ref2.enabled;
      return enabled && onLoad && onLoad(_this.state);
    });
    this.update().then(function () {
      // After the first update completed, enable the event listeners
      _this.enableEventListeners(_this.state.options.eventListeners);
    });
  } // Async and optimistically optimized update
  // it will not be executed if not necessary
  // debounced, so that it only runs at most once-per-tick


  var _proto = Popper.prototype;

  // Syncronous and forcefully executed update
  // it will always be executed even if not necessary, usually NOT needed
  // use Popper#update instead
  _proto.forceUpdate = function forceUpdate() {
    var _this2 = this;

    var _this$state$elements2 = this.state.elements,
        referenceElement = _this$state$elements2.reference,
        popperElement = _this$state$elements2.popper; // Don't proceed if `reference` or `popper` are not valid elements anymore

    if (!areValidElements(referenceElement, popperElement)) {
      return;
    } // Get initial measurements
    // these are going to be used to compute the initial popper offsets
    // and as cache for any modifier that needs them later


    this.state.measures = {
      reference: getElementClientRect(referenceElement),
      popper: getElementClientRect(popperElement)
    }; // Get scrollTop and scrollLeft of the offsetParent
    // this will be used in the `computeOffsets` function to properly
    // position the popper taking in account the scroll position
    // FIXME: right now we only look for a single offsetParent (the popper one)
    //        but we really want to take in account the reference offsetParent as well

    var offsetParentScroll = getNodeScroll(getOffsetParent(popperElement)); // Modifiers have the ability to read the current Popper state, included
    // the popper offsets, and modify it to address specifc cases

    this.state.reset = false; // Cache the placement in cache to make it available to the modifiers
    // modifiers will modify this one (rather than the one in options)

    this.state.placement = this.state.options.placement;
    this.state.orderedModifiers.forEach(function (modifier) {
      return _this2.state.modifiersData[modifier.name] = _extends({}, modifier.data);
    });
    var __debug_loops__ = 0;

    for (var index = 0; index < this.state.orderedModifiers.length; index++) {
      if (process.env.NODE_ENV !== "production") {
        __debug_loops__ += 1;

        if (__debug_loops__ > 100) {
          console.error(INFINITE_LOOP_ERROR);
          break;
        }
      }

      if (index === 0) {
        // Offsets are the actual position the popper needs to have to be
        // properly positioned near its reference element
        // This is the most basic placement, and will be adjusted by
        // the modifiers in the next step
        this.state.offsets = {
          popper: computeOffsets({
            reference: this.state.measures.reference,
            element: this.state.measures.popper,
            strategy: 'absolute',
            placement: this.state.placement,
            scroll: getCommonTotalScroll(referenceElement, this.state.scrollParents.reference, this.state.scrollParents.popper)
          })
        };
      }

      if (this.state.reset === true) {
        this.state.reset = false;
        index = -1;
        continue;
      }

      var _this$state$orderedMo = this.state.orderedModifiers[index],
          fn = _this$state$orderedMo.fn,
          enabled = _this$state$orderedMo.enabled,
          options = _this$state$orderedMo.options;

      if (enabled && typeof fn === 'function') {
        this.state = fn(this.state, options);
      }
    }
  };

  _proto.enableEventListeners = function enableEventListeners(eventListeners) {
    var _this3 = this;

    var _this$state$elements3 = this.state.elements,
        referenceElement = _this$state$elements3.reference,
        popperElement = _this$state$elements3.popper;

    var _ref3 = typeof eventListeners === 'boolean' ? expandToHashMap(eventListeners, ['scroll', 'resize']) : eventListeners,
        scroll = _ref3.scroll,
        resize = _ref3.resize;

    if (scroll) {
      var scrollParents = [].concat(this.state.scrollParents.reference, this.state.scrollParents.popper);
      scrollParents.forEach(function (scrollParent) {
        return scrollParent.addEventListener('scroll', _this3.update, {
          passive: true
        });
      });
    }

    if (resize) {
      var window = getWindow(this.state.elements.popper);
      window.addEventListener('resize', this.update, {
        passive: true
      });
    }
  };

  _proto.destroy = function destroy() {
    var _this4 = this;

    // Remove scroll event listeners
    var scrollParents = [].concat(this.state.scrollParents.reference, this.state.scrollParents.popper);
    scrollParents.forEach(function (scrollParent) {
      return scrollParent.removeEventListener('scroll', _this4.update);
    }); // Remove resize event listeners

    var window = getWindow(this.state.elements.popper);
    window.removeEventListener('resize', this.update); // Run `onDestroy` modifier methods

    this.state.orderedModifiers.forEach(function (_ref4) {
      var onDestroy = _ref4.onDestroy,
          enabled = _ref4.enabled;
      return enabled && onDestroy && onDestroy(_this4.state);
    });
  };

  return Popper;
}();

export { Popper as default };