// This modifier takes the Popper state and prepares some StyleSheet properties
// that can be applied to the popper element to make it render in the expected position.
export var mapStrategyToPosition = function mapStrategyToPosition(strategy) {
  switch (strategy) {
    case 'fixed':
      return 'fixed';

    case 'absolute':
    default:
      return 'absolute';
  }
};
export var computePopperStyles = function computePopperStyles(_ref) {
  var offsets = _ref.offsets,
      strategy = _ref.strategy,
      gpuAcceleration = _ref.gpuAcceleration;

  // by default it is active, disable it only if explicitly set to false
  if (gpuAcceleration === false) {
    return {
      top: offsets.y + "px",
      left: offsets.x + "px",
      position: mapStrategyToPosition(strategy)
    };
  } else {
    return {
      top: '0px',
      left: '0px',
      transform: "translate3d(" + offsets.x + "px, " + offsets.y + "px, 0)",
      position: mapStrategyToPosition(strategy)
    };
  }
};
export var computeArrowStyles = function computeArrowStyles(_ref2) {
  var offsets = _ref2.offsets,
      gpuAcceleration = _ref2.gpuAcceleration;

  if (gpuAcceleration) {
    return {
      top: offsets.y + "px",
      left: offsets.x + "px",
      position: 'absolute'
    };
  } else {
    return {
      transform: "translate3d(" + offsets.x + "px, " + offsets.y + "px, 0)",
      position: 'absolute'
    };
  }
};
export function computeStyles(state, options) {
  if (options === void 0) {
    options = {};
  }

  var _options = options,
      _options$gpuAccelerat = _options.gpuAcceleration,
      gpuAcceleration = _options$gpuAccelerat === void 0 ? true : _options$gpuAccelerat;
  state.styles = {}; // popper offsets are always available

  state.styles.popper = computePopperStyles({
    offsets: state.offsets.popper,
    strategy: state.options.strategy,
    gpuAcceleration: gpuAcceleration
  }); // arrow offsets may not be available

  if (state.offsets.arrow != null) {
    state.styles.arrow = computeArrowStyles({
      offsets: state.offsets.arrow,
      gpuAcceleration: gpuAcceleration
    });
  }

  return state;
}
export default {
  name: 'computeStyles',
  enabled: true,
  phase: 'afterMain',
  fn: computeStyles
};