import getBasePlacement from './getBasePlacement';
import getVariationPlacement from './getVariationPlacement';
import getMainAxisFromPlacement from './getMainAxisFromPlacement';
import { top, right, bottom, left, start, end } from '../enums';
export default (function (_ref) {
  var reference = _ref.reference,
      element = _ref.element,
      strategy = _ref.strategy,
      placement = _ref.placement,
      scroll = _ref.scroll;
  var basePlacement = placement ? getBasePlacement(placement) : null;
  var variationPlacement = placement ? getVariationPlacement(placement) : null;
  var scrollTop = scroll.scrollTop,
      scrollLeft = scroll.scrollLeft;
  var offsets;

  switch (basePlacement) {
    case top:
      offsets = {
        x: reference.x + reference.width / 2 - element.width / 2 - scrollLeft,
        y: reference.y - element.height - scrollTop
      };
      break;

    case bottom:
      offsets = {
        x: reference.x + reference.width / 2 - element.width / 2 - scrollLeft,
        y: reference.y + reference.height - scrollTop
      };
      break;

    case right:
      offsets = {
        x: reference.x + reference.width - scrollLeft,
        y: reference.y + reference.height / 2 - element.height / 2 - scrollTop
      };
      break;

    case left:
      offsets = {
        x: reference.x - element.width - scrollLeft,
        y: reference.y + reference.height / 2 - element.height / 2 - scrollTop
      };
      break;

    default:
      offsets = {
        x: reference.x - scrollLeft,
        y: reference.y - scrollTop
      };
  }

  var mainAxis = basePlacement ? getMainAxisFromPlacement(basePlacement) : null;

  if (mainAxis != null) {
    var len = mainAxis === 'y' ? 'height' : 'width';

    switch (variationPlacement) {
      case start:
        offsets[mainAxis] -= reference[len] / 2 - element[len] / 2;
        break;

      case end:
        offsets[mainAxis] += reference[len] / 2 - element[len] / 2;
        break;

      default:
    }
  }

  return offsets;
});